class Dots {
  ArrayList<Dot> dots;

  float maxSpeed = 3.0;
  float maxForce = 0.2;
  float distBetween = 2.0;
  float separateStrength = 1;
  float edgeStrength = 0.5;
  
  int leftx = 200;
  int rightx = width - 200;
  int topy = 200;
  int bottomy = height - 200;
  int size = 27;

  Dots(int amount) {
    dots = new ArrayList<Dot>();
    for (int i = 0; i < amount; i ++) {
      dots.add(new Dot(random(leftx, rightx), random(topy, bottomy), size));
    }
  }

  void setAmount(int amount) {
    if (amount > dots.size()) {
      int extra = amount - dots.size();
      for (int i = 0; i < extra; i++) {
        dots.add(new Dot(random(leftx, rightx), random(topy, bottomy), size));
      }
    } else {
      int size = dots.size();
      int remove = size - amount;
      for (int i = size - 1; i >= size - 1 - remove; i--) {
        dots.remove(i);
      }
    }
  }

  void update() {
    for (Dot d : dots) {
      forces(d);
      d.update();
      d.display();
    }
  }

  void forces(Dot d) {
    PVector separateForce = separate(d);
    PVector edgeForce = edge(d);
    //PVector avoidForce = avoid(d);

    separateForce.mult(separateStrength);
    edgeForce.mult(edgeStrength);

    d.applyForce(separateForce);
    d.applyForce(edgeForce);
    //d.applyForce(avoidForce);
  }

  PVector separate(Dot d) {
    PVector steer = new PVector();
    int count = 0;
    for (Dot other : dots) {
      float dist = PVector.dist(d.loc, other.loc);
      float rad = d.r + other.r + distBetween;
      if (dist > 0 && dist < rad) {
        steer = PVector.sub(d.loc, other.loc);
        steer.div(pow(PVector.dist(d.loc, other.loc), 2));
        count++;
      }
    }
    if (count > 0) {
      steer.div(count);
      steer.setMag(maxSpeed);
      steer.sub(d.vel);
      steer.limit(maxForce);
    }
    return steer;
  }

  PVector edge(Dot d) {
    noFill();
    stroke(0);
    strokeWeight(2);
    rect(leftx, topy, rightx-leftx, bottomy-topy);
    
    PVector steer = new PVector();
    if (d.loc.x > rightx || d.loc.x < leftx || d.loc.y < topy || d.loc.y > bottomy) {
      PVector middle = new PVector(width/2, height/2);
      steer = PVector.sub(middle, d.loc);
      steer.setMag(maxSpeed);
      steer.sub(d.vel);
      steer.limit(maxForce);
    }
    return steer;
  }

  PVector avoid(Dot d) {
    PVector steer = new PVector();
    PVector mouse = new PVector(mouseX, mouseY);
    float dist = PVector.dist(d.loc, mouse);
    if (dist < 100) {
      steer = PVector.sub(d.loc, mouse);
    }
    steer.setMag(1);
    return steer;
  }
}
